'Title..... QB4plus
'Function.. Quick Library using DOS/BIOS interrupts
'Author.... Jim Woodruff

DECLARE SUB GetDate (Month%, Day%, Year%, DOW%)
DECLARE SUB FlushKbd ()
DECLARE SUB ScrollDown (URow%, LCol%, LRow%, RCol%, HowMany%, Colour%)
DECLARE SUB ScrollUp (URow%, LCol%, LRow%, RCol%, HowMany%, Colour%)
DECLARE SUB NewDrive (Drive%)
DECLARE SUB DOSver (Major%, Minor%)
DECLARE SUB ClrEOL (Colour%)
DECLARE SUB ClrEOP (Colour%)
DECLARE SUB CRTMode (VideoMode%, Columns%, ActPage%)

DECLARE FUNCTION ShiftStatus% ()
DECLARE FUNCTION ComStatus% (PortNumber%)
DECLARE FUNCTION CurrentDir$ ()
DECLARE FUNCTION CheckBit% (Target%, Bit%)
DECLARE FUNCTION DefaultDrive% ()
DECLARE FUNCTION PrinterErr% (PrnNum%)
DECLARE FUNCTION FreeDiskSpace& ()
DECLARE FUNCTION BaseRam% ()
DECLARE FUNCTION FileExist% (File$)

TYPE RegType
     ax    AS INTEGER
     bx    AS INTEGER
     cx    AS INTEGER
     dx    AS INTEGER
     bp    AS INTEGER
     si    AS INTEGER
     di    AS INTEGER
     flags AS INTEGER
END TYPE

DIM SHARED InRegs AS RegType, OutRegs AS RegType

FUNCTION BaseRam%
    CALL interrupt(&H12, InRegs, OutRegs)
    BaseRam% = OutRegs.ax
END FUNCTION

FUNCTION CheckBit% (Target%, Bit%)
    CheckBit% = (Target% AND 2 ^ Bit%)
END FUNCTION

SUB ClrEOL (Colour%)
    IF Colour% < 0 OR Colour% > 7 THEN Colour% = 0
    Colour% = Colour% * 16
    CALL CRTMode(VidMode%, Columns%, ActPage%)
    InRegs.ax = &H920
    InRegs.bx = ActPage% * 256 + Colour%
    InRegs.cx = Columns% - POS(44) + 1
    CALL interrupt(&H10, InRegs, OutRegs)
END SUB

SUB ClrEOP (Colour%)
    IF Colour% < 0 OR Colour% > 7 THEN Colour% = 0
    Colour% = Colour% * 16
    CALL CRTMode(VidMode%, Columns%, ActPage%)
    InRegs.ax = &H920
    InRegs.bx = ActPage% * 256 + Colour%
    InRegs.cx = (Columns% - POS(44) + 1) + (25 - CSRLIN) * 80
    CALL interrupt(&H10, InRegs, OutRegs)
END SUB

FUNCTION ComStatus% (PortNumber%)
    IF PortNumber% < 1 OR PortNumber% > 4 THEN EXIT FUNCTION
    InRegs.ax = &H300
    InRegs.dx = PortNumber% - 1
    CALL interrupt(&H14, InRegs, OutRegs)
    ComStatus% = InRegs.ax
END FUNCTION

SUB CRTMode (VideoMode%, Columns%, ActPage%)
    InRegs.ax = &HF00
    CALL interrupt(&H10, InRegs, OutRegs)
    VideoMode% = OutRegs.ax MOD 256
    Columns% = OutRegs.ax \ 256
    ActPage% = OutRegs.bx \ 256
END SUB

FUNCTION CurrentDir$
    InRegs.ax = &H4700
    InRegs.dx = &H0
    Dir$ = STRING$(64, 32)
    InRegs.si = SADD(Dir$)
    CALL interrupt(&H21, InRegs, OutRegs)
    CurrentDir$ = CHR$(65 + DefaultDrive%) + ":\" + RTRIM$(Dir$)
END FUNCTION

FUNCTION DefaultDrive%
    InRegs.ax = &H1900
    CALL interrupt(&H21, InRegs, OutRegs)
    DefaultDrive% = OutRegs.ax MOD 256
END FUNCTION

SUB DOSver (Major%, Minor%)
    InRegs.ax = &H3000
    CALL interrupt(&H21, InRegs, OutRegs)
    Major% = OutRegs.ax MOD 256
    Minor% = OutRegs.ax \ 256
END SUB

FUNCTION FileExist% (File$)
    InRegs.ax = &H3D00
    File$ = File$ + CHR$(0)
    InRegs.dx = SADD(File$)
    CALL interrupt(&H21, InRegs, OutRegs)
    File$ = LEFT$(File$, LEN(File$) - 1)
    IF (OutRegs.flags AND 1) THEN FileExist% = 0: EXIT FUNCTION
    InRegs.ax = &H3E00
    InRegs.bx = OutRegs.ax
    CALL interrupt(&H21, InRegs, OutRegs)
    IF (OutRegs.flags AND 1) THEN FileExist% = 0: EXIT FUNCTION
    FileExist% = -1
END FUNCTION

SUB FlushKbd
    InRegs.ax = &HC00
    CALL interrupt(&H21, InRegs, OutRegs)
END SUB

FUNCTION FreeDiskSpace&
    InRegs.ax = &H3600
    InRegs.dx = 0
    CALL interrupt(&H21, InRegs, OutRegs)
    bytes& = (OutRegs.cx * OutRegs.ax)
    FreeDiskSpace& = bytes& * OutRegs.bx
END FUNCTION

SUB GetDate (Month%, Day%, Year%, DOW%)
    InRegs.ax = &H2A00
    CALL interrupt(&H21, InRegs, OutRegs)
    Month% = OutRegs.dx \ 256
    Day% = OutRegs.dx MOD 256
    Year% = OutRegs.cx
    DOW% = OutRegs.ax MOD 256
END SUB

SUB NewDrive (Drive%)
    InRegs.ax = &HE00
    InRegs.dx = Drive%
    CALL interrupt(&H21, InRegs, OutRegs)
END SUB

FUNCTION PrinterErr% (PrnNum%)
    InRegs.ax = &H200
    InRegs.dx = PrnNum% - 1
    CALL interrupt(&H17, InRegs, OutRegs)
    PE% = OutRegs.ax \ 256
    IF PE% <= 0 THEN PrinterErr% = 0 ELSE PrinterErr% = PE%
END FUNCTION

SUB ScrollDown (URow%, LCol%, LRow%, RCol%, HowMany%, Colour%)
    IF Colour% < 0 OR Colour% > 7 THEN Colour% = 0
    Colour% = Colour% * 16
    URow% = URow% - 1
    LCol% = LCol% - 1
    LRow% = LRow% - 1
    RCol% = RCol% - 1
    InRegs.ax = 1792 + HowMany%
    InRegs.bx = Colour% * 256
    InRegs.cx = URow% * 256 + LCol%
    InRegs.dx = LRow% * 256 + RCol%
    CALL interrupt(&H10, InRegs, OutRegs)
END SUB

SUB ScrollUp (URow%, LCol%, LRow%, RCol%, HowMany%, Colour%)
    IF Colour% < 0 OR Colour% > 7 THEN Colour% = 0
    Colour% = Colour% * 16
    URow% = URow% - 1
    LCol% = LCol% - 1
    LRow% = LRow% - 1
    RCol% = RCol% - 1
    InRegs.ax = 1536 + HowMany%
    InRegs.bx = Colour% * 256
    InRegs.cx = URow% * 256 + LCol%
    InRegs.dx = LRow% * 256 + RCol%
    CALL interrupt(&H10, InRegs, OutRegs)
END SUB

FUNCTION ShiftStatus%
    InRegs.ax = &H200
    CALL interrupt(&H16, InRegs, OutRegs)
    ShiftStatus% = OutRegs.ax MOD 256
END FUNCTION

